<?php

/**
 * @file
 * A collection of large, static strings--such as Javascript and HTML code--in MM
 */

/**
 * Return (or add to the current page) some Javascript/HTML/text used in MM
 *
 * @param $which
 *   Key of the code to return/add
 * @param $add_js
 *   If TRUE, add the code to the current page with drupal_add_js(). If
 *   'return_once', only return the code if it is not already present.
 * @param $additional
 *   Any additional parameters are passed to the underlying function
 * @return
 *   The Javascript code
 */
function mm_static($which, $add_js = FALSE) {
  $added = &drupal_static(__FUNCTION__, array());

  $args = func_get_args();
  array_splice($args, 0, 2);
  // While this could be done with an array, that would be less efficient.
  // Instead, require that a function of the format "_mm_static__$which" exists
  // and returns some text.
  $name = "_mm_static__$which";
  if (!function_exists($name)) return;
  $code = call_user_func_array($name, $args);
  if (!empty($code)) {
    // If the code uses jQuery, add a prefix and suffix to initialize "$".
    if (strpos($code, '$(') !== FALSE) {
      $code = "(function (\$) {\n{$code}})(jQuery);";
    }

    if ($add_js && empty($added[$which])) {
      if ($add_js === TRUE) {
        drupal_add_js($code, array('type' => 'inline', 'scope' => 'footer'));
      }
      $added[$which] = TRUE;
    }
    else if ($add_js === 'return_once') {
      return '';
    }
  }
  return $code;
}

/* Referenced by mm_ui_content_edit, _mm_ui_node_form_perms */
function _mm_static__settings_perms($form_id = 'mm-ui-content-edit', $submit_id = 'edit-submit', $row_selector = MM_PERMS_READ) {
  drupal_add_library('monster_menus', 'mm_settings_perms');
  drupal_add_js(array('mm_settings_perms' => array(
    'r'                 => MM_PERMS_READ,
    'w'                 => MM_PERMS_WRITE,
    'row_selector'      => $row_selector,
    'disabled_selector' => $row_selector == MM_PERMS_READ ? MM_PERMS_WRITE : $row_selector,
    'form_id'           => $form_id,
    'submit_id'         => $submit_id,
  )), array('type' => 'setting'));
}

/* Referenced by mm_node_redir_form */
function _mm_static__node_redir() {
  return <<<JS
Drupal.behaviors.MMNodeRedir = {
  attach: function (context, settings) {
    \$('.node-redir-url', context).once('node-redir-url', function () {
      \$(this).change(function () {
        if (this.value != '' && this.form['redir_mmtid'].value != '')
          this.form['redir_mmtid'].delAll();
      });
    });
  }
};
JS;
}

/* Referenced by mm_node_alert_add_to_footer */
function _mm_static__node_alert() {
  mm_ui_modal_dialog();
  return <<<JS
Drupal.behaviors.MMNodeAlert = {
  attach: function (context, settings) {
    // Intentionally don't use 'once' here
    $('.mm-alert:not(.mm-alert-processed):first', context)
      .addClass('mm-alert-processed')
      .each(function () {
        $('.mm-alert-close :button', this).click(function () {
          Drupal.mmDialogClose();
          if ($('.mm-alert:not(.mm-alert-processed)').length) {
            Drupal.behaviors.MMNodeAlert.attach(context); // show next alert while reappearing
          }
          return false;
        });
        Drupal.mmDialogAdHoc('#' + this.id, '', {
          width: 300,
          height: 300,
          show: {
            effect: "fadeIn",
            duration: "fast"
          }, 
          hide: {
            effect: "fadeOut", 
            duration: "fast"
          }
        });
      });
  }
};
JS;
}

/* Referenced by mm_ui_content_copymove */
function _mm_static__copymove($mustcheck) {
  drupal_add_js(array('MM' => array('copymove' => array('mustcheck' => $mustcheck))), array('type' => 'setting'));
  drupal_add_js(drupal_get_path('module', 'monster_menus') . '/js/copymove.js');
}

/* Referenced by theme_mm_archive */
function _mm_static__select_menu() {
  drupal_add_js(drupal_get_path('module', 'monster_menus') . '/js/select_menu.js');
}

/* Referenced by mm_ui_solver */
function _mm_static__solver_mm_list_callback($mmtid) {
  drupal_add_js(array('MM' => array('solver_mm_list_callback' => array('path' => base_path() . "mm/$mmtid/settings/solver/"))), array('type' => 'setting'));
  drupal_add_js(drupal_get_path('module', 'monster_menus') . '/js/solver_mm_list_callback.js');
}

/* Referenced by mm_browser_bookmark_add */
function _mm_static__mm_browser_bookmark_add($name, $mmtid) {
  global $base_url;

  $base = base_path();
  $mm_path = $base . drupal_get_path('module', 'monster_menus');
  $title = t('Title:');
  $add = t('Add');
  $cancel = t('Cancel');
  $name = check_plain($name);
  $title_msg = t('Please enter a title for this bookmark.');
  return <<<HTML
<html><head>
  <link rel="stylesheet" href="$mm_path/css/mm_browser_tb.css" />
  <script type="text/javascript" src="$base_url/misc/jquery.js"></script>
</head>
<body id="tb-body">
  <div id="message"><p>$title_msg</p></div>
  <div id="add-bookmark-div">
    <form action="#" onsubmit="return Drupal.mm_browser_add_bookmark_submit(document);">
      <table border="0" cellpadding="3" cellspacing="3" style="margin:0 auto;" >
        <tr>
          <td><label>$title</label></td>
          <td>
            <input name="linktitle" id="linktitle" type="text" size="25" maxlength="35" value="$name" tabindex="1">
            <input name="linkmmtid" type="hidden" value="$mmtid">
          </td>
        </tr>
        <tr align="right">
          <td colspan="2">
            <input type="submit" id="Add" value="$add" onclick="return Drupal.mm_browser_add_bookmark_submit(document);" tabindex="2">&nbsp;
            <input type="submit" id="Cancel" value="$cancel" onclick="Drupal.mmDialogClose(); return false;" tabindex="3">
          </td>
        </tr>
      </table>
    </form>
  </div>
</body></html>
HTML;
}

/* Referenced by mm_browser_bookmark_manage */
function _mm_static__mm_browser_bookmark_manage() {
  global $base_url;

  return <<<JS
\$(function () {
  \$("#sortable").sortable({
    axis:                 'y',
    containment:          'document',
    placeholder:          'ui-state-highlight',
    forcePlaceholderSize: true,
    opacity:              0.5,
    stop:                 function (event, ui) {
      var serialStr = "";
      \$("ul#sortable li").each(function () {
        serialStr += \$(this).attr("name") + "|";
      });
      \$.post(Drupal.mm_browser_append_params("$base_url/mm-bookmarks/sort"),
        {neworder: serialStr.substring(0, serialStr.length - 1)},
        function () {
          Drupal.mm_browser_get_bookmarks();
        },
        "json");
    }
  });
  \$("#sortable").disableSelection();
});
JS;
}

function _mm_static__back_in_history() {
  return <<<JS
Drupal.mm_back_in_history = function (save) {
  if (save) {
    document.cookie = 'mm_last_page=' + encodeURI(document.location) + ';path=/;SameSite=Strict';
  }
  else {
    var matches = document.cookie.match(/\bmm_last_page=(.*?)(;|$)/);
    if (matches && matches.length && matches[1].length) {
      var date = new Date(0);
      document.cookie = "mm_last_page=;expires=" + date.toUTCString() + ";path=/;SameSite=Strict";
      document.cookie = 'goto_last=1;path=/;SameSite=Strict';
      document.location = matches[1];
    }
    else {
      document.cookie = 'goto_last=1;path=/;SameSite=Strict';
      window.history.back(-1);
    }
    return false;
  }
};
JS;
}

/* Referenced by _mm_ui_content_edit */
function _mm_static__ui_content_edit_js($mmtid, $token, $element_name, $is_new) {
  drupal_add_js(drupal_get_path('module', 'monster_menus') . '/js/group_table.js');

  $columns = _mm_ui_userlist_get_headers();
  $col_def = array_fill(0, count($columns) - 1, NULL);
  $col_def[] = array('bSortable' => FALSE);

  $new = $is_new ? '/new' : '';
  $settings = array(
    'clean_element' => str_replace('-', '_', $element_name),
    'id_name'       => str_replace('_', '-', $element_name),
    'col_def'       => $col_def,
    'load'          => "-mm-users-json/$mmtid/$token/listusers/$element_name$new",
    'delete'        => "-mm-users-json/$mmtid/$token/deleteusers/-USER-$new",
    'add'           => "-mm-users-json/$mmtid/$token/addusers/-UIDS-$new",
  );
  drupal_add_js(array('MM' => array('group_table' => $settings)), array('type' => 'setting'));
}

function _mm_static__lazy_load_node($mmtid, $max_pages) {
  if ($max_pages) {
    $mm = drupal_get_path('module', 'monster_menus');
    drupal_add_js("$mm/js/lazy_load_node.js");
    drupal_add_js(array('MM' => array(
      'lazy_load_node' => array(
        'loading_img' => url("$mm/images/ajax-loader.gif"),
        'mmtid' => $mmtid,
        'max_pages' => $max_pages,
      ))), array('type' => 'setting'));
  }
}

/* Referenced by mm_show_group */
function _mm_static__show_group($mmtid, $col_def) {
  $link_location = url('-mm-users-json/' . $mmtid . '/listusers/mm-user-datatable-members-display');
  $count_text = t('Showing _START_ to _END_ of _TOTAL_ record(s)');
  $filtered_text = t('(filtered from _MAX_ total record(s))');
  return <<<JS
Drupal.behaviors.MMShowGroup = {
  attach: function () {
    var initialResize = 0;
    \$('#mm-user-datatable-members-display').dataTable({
      aoColumns:      [$col_def],
      bProcessing:    true,
      bServerSide:    true,
      iDisplayLength: 20,
      bLengthChange:  false,
      sAjaxSource:    '$link_location',
      oLanguage:      {sInfo: "$count_text", sInfoFiltered: "$filtered_text"},
      fnDrawCallback: function() {
                        if (!initialResize++) {
                          parent.Drupal.mmDialogResized(document.body.scrollWidth, document.body.scrollHeight);
                        }
                      }
    });
  }
};
JS;
}

function _mm_static__mm_ui_node_reorder() {
  /**
   * Move a block in the blocks table from one region to another via select list.
   *
   * This behavior is dependent on the tableDrag behavior, since it uses the
   * objects initialized in that behavior to update the row.
   */
  return <<<JS
Drupal.behaviors.nodeReorderDrag = {
  attach: function (context, settings) {
    // tableDrag is required and we should be on the blocks admin page.
    if (typeof Drupal.tableDrag == 'undefined' || typeof Drupal.tableDrag['mm-reorder1'] == 'undefined' && typeof Drupal.tableDrag['mm-reorder2'] == 'undefined') {
      return;
    }

    var table = $('table#mm-reorder1,table#mm-reorder2');
    var tableDrag1 = Drupal.tableDrag['mm-reorder1'] || {row: {prototype: {}}}; // Sticky tableDrag object.
    var tableDrag2 = Drupal.tableDrag['mm-reorder2'] || {row: {prototype: {}}}; // Unsticky tableDrag object.

    // Add a handler for when a row is swapped, update empty regions.
    tableDrag1.row.prototype.onSwap = tableDrag2.row.prototype.onSwap = function (swappedRow) {
      var rowObject = this;
      $('tr.region-message', table).each(function () {
        // If the dragged row is in this region, but above the message row, swap it down one space.
        if ($(this).prev('tr').get(0) == rowObject.element) {
          // Prevent a recursion problem when using the keyboard to move rows up.
          if ((rowObject.method != 'keyboard' || rowObject.direction == 'down')) {
            rowObject.swap('after', this);
          }
        }
        // This region has become empty.
        if ($(this).next('tr').is(':not(.draggable)') || $(this).next('tr').length == 0) {
          $(this).removeClass('region-populated').addClass('region-empty');
        }
        // This region has become populated.
        else if ($(this).is('.region-empty')) {
          $(this).removeClass('region-empty').addClass('region-populated');
        }
      });
    };

    tableDrag1.row.prototype.isValidSwap = tableDrag2.row.prototype.isValidSwap = function (row) {
      return !$(row).is('.mm-not-draggable') && !$(row).is('.region-title') && (row != this.table.tBodies[0].rows[0] || $(row).is('.draggable'));
    };

    // A custom message for the blocks page specifically.
    Drupal.theme.tableDragChangedWarning = function () {
      return '<div class="messages warning">' + Drupal.theme('tableDragChangedMarker') + ' ' + Drupal.t('The changes to this table will not be saved until the <em>Save configuration</em> button is clicked.') + '</div>';
    };

    tableDrag1.onDrag = tableDrag2.onDrag = function () {
      var dragObject = this;
      var regionField = $('select.nodes-region-select', dragObject.rowObject.element);
      $(this.rowObject.element).siblings('tr.draggable,tr.region-message').each(function() {
        var regionRow = $(this).is('.region-message') ? this : $(this).prevAll('tr.region-message').get(0);
        var regionName = regionRow.className.replace(/([^ ]+[ ]+)*region-([^ ]+)-message([ ]+[^ ]+)*/, '$2');
        if ($('option[value=' + regionName + ']', regionField).length == 0) {
          $(regionRow).prev().add(this).addClass('mm-not-draggable');
        }
      });
    };

    // Add a handler so when a row is dropped, update fields dropped into new regions.
    tableDrag1.onDrop = tableDrag2.onDrop = function () {
      $(this.rowObject.element).siblings('tr.mm-not-draggable').removeClass('mm-not-draggable');
      var dragObject = this;
      // Use "region-message" row instead of "region" row because
      // "region-{region_name}-message" is less prone to regexp match errors.
      var regionRow = $(dragObject.rowObject.element).prevAll('tr.region-message').get(0);
      var regionName = regionRow.className.replace(/([^ ]+[ ]+)*region-([^ ]+)-message([ ]+[^ ]+)*/, '$2');
      var regionField = $('select.nodes-region-select', dragObject.rowObject.element);
      // Check whether the newly picked region is available for this node.
      if ($('option[value=' + regionName + ']', regionField).length == 0) {
        // If not, alert the user and keep the node in its old region setting.
        alert(Drupal.t('The content cannot be placed in this region.'));
        // Simulate that there was a selected element change, so the row is put
        // back to from where the user tried to drag it.
        regionField.change();
      }
      else if ($(dragObject.rowObject.element).prev('tr').is('.region-message')) {
        var weightField = $('select.nodes-weight', dragObject.rowObject.element);
        var oldRegionName = weightField[0].className.replace(/([^ ]+[ ]+)*nodes-weight-([^ ]+)([ ]+[^ ]+)*/, '$2');

        if (!regionField.is('.nodes-region-' + regionName)) {
          regionField.removeClass('nodes-region-' + oldRegionName).addClass('nodes-region-' + regionName);
          weightField.removeClass('nodes-weight-' + oldRegionName).addClass('nodes-weight-' + regionName);
          regionField.val(regionName);
        }
      }
    };
  }
};
JS;
}

