<?php

/**
 * @file
 * Functions used by MM Media Assist
 */

/**
 * Print the main mm_media_assist category browser.
 */
function mm_media_assist_loader($browser_type, $media_types = NULL, $media_min_wh = '0x0') {
  module_load_include('inc', 'monster_menus', 'mm_browser');
  $file_types = NULL;
  $min_wh = NULL;
  if ($browser_type == MM_BROWSER_MODE_WYSIWYG) {
    $location = isset($_GET['location']) ? $_GET['location'] : '';
    $end = 0;
    if (stripos($location, '/node/') > 1) {
      $end = stripos($location, '/node/');
    }
    if (stripos($location, '/contents/') > 1) {
      $end = stripos($location, '/contents/');
    }
    $myterm_ids = array();
    if ($end) {
      $start = stripos($location, base_path()) + strlen(base_path());
      $url = substr($location, $start, $end - $start);
      monster_menus_url_inbound_alter($result, $url, NULL);
      mm_parse_args($myterm_ids, $oarg_list, $this_tid, $result);
    }
    $top_mmtid = isset($myterm_ids[0]) ? $myterm_ids[0] : 1;
    $start = isset($this_tid) && is_numeric($this_tid) ? $this_tid : mm_home_mmtid();
  }
  else {
    $term_ids = func_get_args();
    array_shift($term_ids);
    if ($browser_type == MM_BROWSER_MODE_MEDIA) {
      $file_types = is_numeric($media_types) ? array_keys(file_entity_type_get_names()) : explode(',', $media_types);
      $min_wh = explode('x', $media_min_wh);
      array_shift($term_ids);
      array_shift($term_ids);
    }
    $dashed = array_shift($term_ids);
    array_shift($term_ids);
    $start = array_pop($term_ids);
    if (!mm_media_sanitize_termids($term_ids)) {
      // Sanitization failed.
      return;
    }
    if ($term_ids) {
      $top_mmtid = $term_ids[0];
    }
    else {
      $dashed = explode('-', $dashed);
      $top_mmtid = $dashed[0];
    }
  }

  print theme('mm_media_assist_page',
    array(
      'content' => mm_browser($start, $top_mmtid, $browser_type, MM_PERMS_READ, '', '', NULL, $file_types, $min_wh),
      'attributes' => array(
        'id' => 'mm-media-assist-load',
        'class' => array('mm-media-assist'),
      )
    )
  );
  $GLOBALS['devel_shutdown'] = FALSE; // prevent the devel module from outputting
  exit;
}

/**
 * Load the media properties pane
 */
function mm_media_assist_properties($file) {
  drupal_add_library('system', 'ui');

  $form = drupal_get_form('mm_media_assist_properties_form', $file);
  $output = drupal_render($form);
  print theme('mm_media_assist_page',
    array(
      'content' => $output,
      'attributes' => array(
        'id' => 'mm-media-assist-properties',
        'class' => array('mm-media-assist'),
      )
    )
  );
  $GLOBALS['devel_shutdown'] = FALSE; // prevent the devel module from outputting
  exit;
}

function mm_media_assist_properties_form($form, $form_state, $file) {
  module_load_include('inc', 'media', 'includes/media.filter');
  $form[] = array('#markup' => '<div id="mm-media-assist"><div id="mm-media-assist-nav"><h2 class="mm-media-assist-title-nomenu">' . t('File Properties') . '</h2></div>');

  $title = $file->filename;
  if (!empty($file->title)) {
    $title = $file->title;
  }
  else if (!empty($file->description)) {
    $title = $file->description;
  }

  $form_title = array(
    '#type' => 'textfield',
    '#title' => t('Title (optional)'),
    '#default_value' => strip_tags($title),
    '#size' => 30,
    '#maxlength' => 255,
  );

  // Buttons
  $actions = array(
    // Can't use '#type' => 'actions' here, because it messes up the weights
    '#prefix' => '<div id="edit-actions" class="form-actions form-wrapper">',
    '#suffix' => '</div>',
  );
  $actions['insert'] = array(
    '#type' => 'button',
    '#value' => t('Insert'),
    '#button_type' => 'button',
  );
  $actions['cancel'] = array(
    '#type' => 'button',
    '#value' => t('Cancel'),
    '#button_type' => 'button',
  );
  $actions['abort'] = array(
    '#type' => 'button',
    '#value' => t('Close window'),
    '#button_type' => 'button',
  );

  if ($file->type == 'image') {
    $entity_info = entity_get_info('file');
    $view_modes = $entity_info['view modes'];
    $options = $formats = array();
    foreach ($view_modes as $view_mode => $view_mode_info) {
      // Don't present the user with an option to choose a view mode in which the
      // file is hidden.
      $extra_fields = field_extra_fields_get_display('file', $file->type, $view_mode);
      if (isset($extra_fields['file']['visible']) && !$extra_fields['file']['visible']) {
        continue;
      }

      $options[$view_mode] = $view_mode_info['label'];
      $element = function_exists('media_wysiwyg_get_file_without_label') ? media_wysiwyg_get_file_without_label($file, $view_mode, array('wysiwyg' => TRUE)) : media_get_file_without_label($file, $view_mode, array('wysiwyg' => TRUE));

      // Make a pretty name out of this.
      $formats[$view_mode] = drupal_render($element);
    }

    if (!$formats) {
      throw new Exception('Unable to continue, no available formats for displaying media.');
    }

    $default_view_mode = variable_get('media__wysiwyg_default_view_mode');
    if (!isset($formats[$default_view_mode])) {
      $default_view_mode = key($formats);
    }

    // Create the form
    $form[] = array(
      '#markup' => '<div id="mm-media-assist-form" class="mm-media-assist-table"><p>' . t('Click "Insert" to embed this image in your text.') . '</p>');

    $form['title'] = $form_title;
    $form['title']['#description'] = t('The title shows when the mouse hovers over the image.');
    $form['desc'] = array(
      '#type' => 'textfield',
      '#title' => t('Alternate text (optional)'),
      '#default_value' => $file->alt,
      '#size' => 30,
      '#maxlength' => 255,
      '#description' => t('Alternate text won\'t show onscreen, but will be read by assistive devices for the blind.'),
    );

    // Presentation Style
    $form['format'] = array(
      '#title' => t('Style'),
      '#type' => 'select',
      '#default_value' => $default_view_mode,
      '#options' => $options,
    );

    // Settings
    $form[] = array('#markup' => '<div class="form-item" id="link-group">', '#title' => t('Settings'));
    $form['mm_media_link'] = array('#type' => 'checkbox', '#title' => t('Clicking on the image shows it full-size'));
    $form['mm_media_outlined'] = array('#type' => 'checkbox', '#title' => t('Show a black border around the image'));
    $form['mm_media_margin'] = array('#type' => 'checkbox', '#title' => t('Include some empty space around the image'));
    $form['actions'] = $actions;
    $form[] = array('#markup' => '</div>');

    $form[] = array('#markup' => "</div><div id=\"mm-media-assist-preview\"><div class=\"mm-media-browse-imagesingle ui-state-default ui-widget-content\"><div class=\"ui-widget-header mm-media-browse-caption\">");
    $form[] = array('#markup' => t('Preview of %title', array('%title' => $title)));
    $form[] = array('#markup' => '</div><div id="mm-media-preview"></div>');
    $form[] = array('#markup' => '</div></div>');

    drupal_add_js(array('mm_media' => array('formatFormFormats' => $formats)), 'setting');
  }   // end isimage
  else {
    $form[] = array(
      '#markup' => '<div id="mm-media-assist-form" class="mm-media-assist-table"><p>' . t('Click "Insert" to embed this file upload in your text.') . '</p><p>' . t('The Description will appear next to the link to the file.') . '</p>');
    $properties_image = theme('mm_browser_thumbnail', array('file' => $file, 'mode' => ''));

    $form['title'] = $form_title;
    $form['desc'] = array(
      '#type' => 'textfield',
      '#title' => t('Description (optional)'),
      '#default_value' => strip_tags($title),
      '#size' => 30,
      '#maxlength' => 255,
    );

    if ($file->type == 'audio') {
      $form['video'] = array(
        '#type' => 'hidden',
        '#value' => $file->uri,
      );
      $form['vwidth'] = array(
        '#type' => 'hidden',
        '#value' => 300,
      );
      $form['vheight'] = array(
        '#type' => 'hidden',
        '#value' => 24,
      );
      $form['autoplay'] = array(
        '#type' => 'checkbox',
        '#default_value' => 0,
        '#title' => t('Auto start audio'),
        '#description' => NULL,
      );
    }
    elseif ($file->type == 'video' || $file->filemime == 'application/x-shockwave-flash') {
      $form['video'] = array(
        '#type' => 'hidden',
        '#value' => $file->uri,
      );
      if (module_exists('getid3')) {
        $info = getid3_analyze(drupal_realpath($file->uri));
      }
      $form['vwidth'] = array(
        '#type' => 'textfield',
        '#size' => 25,
        '#default_value' => (isset($info['video']['resolution_x']) ? $info['video']['resolution_x'] : ''),
        '#title' => 'Video width',
      );
      $form['vheight'] = array(
        '#type' => 'textfield',
        '#size' => 25,
        '#default_value' => (isset($info['video']['resolution_y']) ? $info['video']['resolution_y'] : ''),
        '#title' => 'Video height',
      );
      $form['bg'] = array(
        '#type' => 'textfield',
        '#size' => 45,
        '#maxlength' => 400,
        '#default_value' => '',
        '#title' => t('Full URL for the preview image (optional)'),
        '#description' => t('Provide node id for thumbnail e.g. http://hostname.com/system/files/styles/preview/private/filename.png'),
      );
      $form['autoplay'] = array(
        '#type' => 'checkbox',
        '#default_value' => 0,
        '#title' => t('Auto start movie'),
        '#description' => NULL,
      );
    }

    if ($file->filemime == 'application/x-director') {
      $form['sound'] = array(
        '#type' => 'checkbox',
        '#title' => t('Sound'),
        '#default_value' => 1,
        '#description' => NULL,
        '#prefix' => '<div class="container-inline">'
      );
      $form['autostart'] = array(
        '#type' => 'checkbox',
        '#title' => t('Autostart'),
        '#default_value' => 1,
        '#description' => NULL,
        '#prefix' => '<span class="rightalign">',
        '#suffix' => '</span></div>'
      );
      $form['progress'] = array(
        '#type' => 'checkbox',
        '#title' => t('Progress'),
        '#description' => NULL,
        '#prefix' => '<div class="container-inline">'
      );
      $form['swliveconnect'] = array(
        '#type' => 'checkbox',
        '#title' => t('SWLiveConnect'),
        '#prefix' => '<span class="rightalign">',
        '#suffix' => '</span></div>'
      );
    }

    $form['actions'] = $actions;
    $form[] = array('#markup' => "</div><div id=\"mm-media-assist-preview\"><div class=\"mm-media-browse-imagesingle ui-state-default ui-widget-content\"><div class=\"ui-widget-header mm-media-browse-caption\">");
    $form[] = array('#markup' => t('Preview of %title', array('%title' => $title)) . '</div>');
    $form[] = array('#markup' => '<div id="mm-media-preview">' . $properties_image . '</div>');
    $form[] = array('#markup' => '</div></div>');
  }
  // Hidden Fields
  $form['fid'] = array(
    '#type' => 'hidden',
    '#value' => $file->fid,
  );
  $form['instanceId'] = array(
    '#type' => 'hidden',
    '#value' => empty($_GET['instanceId']) ? 0 : $_GET['instanceId'],
  );

  $form['#attributes'] = array('name' => 'mm_media_assist', 'class' => array('mm-media-assist'));
  return $form;
}

// --------------------------------------------------------------------------------
// Theme Functions
// --------------------------------------------------------------------------------

/**
 * Theme the media assist page
 *
 * @param $variables
 *   An associative array containing:
 *   - content: the selected node
 *   - attributes: any attributes
 * @return string
 *   HTML code
 * @ingroup themeable
 */
function theme_mm_media_assist_page($variables) {
  drupal_add_css(drupal_get_path('module', 'mm_media') . '/css/mm_media_assist.css');
  _mm_media_assist_add_custom_js();
  return theme('mm_page_wrapper',
    array(
      'title' => t('Select a file upload'),
      'body' => $variables['content'],
      'wrap_body_with_div' => FALSE,
      'attributes' => $variables['attributes']
    )
  );
}

function _mm_media_assist_add_custom_js() {
  drupal_add_js(mm_static('back_in_history') . <<<JS
(function ($) {

Drupal.mmMediaUpdateCaption = function() {
  var title = $("input[name=title]").val().trim();
  if (title != "") {
    $("#mm-media-preview img").attr("title", title);
  }
  else {
    $("#mm-media-preview img").removeAttr("title");
  }
  var alt = $("input[name=desc]").val().trim();
  if (alt != "") {
    $("#mm-media-preview img").attr("alt", alt);
  }
  else {
    $("#mm-media-preview img").removeAttr("alt");
  }
};

Drupal.mmMediaInsertImage = function() {
  if (window.parent) {
    // Get variables from the fields on the properties frame
    var media = {
      options: {},
      type: $("select#edit-format").val() || "media",
      html: $("#mm-media-preview").html() || $("<img />"),
    };
    $(":input:not(:submit)").each(function(i, field) {
      if (field.name != "instanceId" && field.name.substr(0, 5) != "form_" && ($(this).attr("type") != "checkbox" || $(this).attr("checked")))
        media.options[field.name] = $(this).val();
    });
    window.parent.Drupal.wysiwyg.plugins.mm_media.insertMediaFile(
      {fid: $("input[name=fid]").val()},
      media.type, media.html, media.options,
      window.parent.Drupal.wysiwyg.instances[$("input[name=instanceId]").val()]);
      window.parent.Drupal.mmDialogClose();
  }
  else {
    alert(Drupal.t('The media cannot be inserted because the page that opened the popup is no longer present.'));
  }
  return false;
};

Drupal.mmMediaBrowserAddFile = function(fid) {
  window.location = Drupal.settings.basePath + "media/browser?render=media-popup&fid=" + fid;
};

Drupal.behaviors.mmMediaInit = {
  attach: function (context, settings) {
    $(".mm-media-assist-table", context).once("mm-media-assist-table", function () {
      $("input[name=title],input[name=desc]")
        .blur(Drupal.mmMediaUpdateCaption);
      Drupal.mmMediaUpdateCaption();
      $("input#edit-abort")
        .click(function() {
          window.parent.Drupal.mmDialogClose();
          return false;
        });
      $("input#edit-cancel")
        .click(function() {
          Drupal.mm_back_in_history();
          return false;
        });
      $("input#edit-insert")
        .click(function() {
          Drupal.mmMediaUpdateCaption();
          Drupal.mmMediaInsertImage();
        });
      $("select#edit-format")
        .change(function() {
          $("#mm-media-preview").html(Drupal.settings.mm_media.formatFormFormats[$(this).val()]);
        })
        .trigger("change");
      $("input#edit-mm-media-outlined")
        .change(function() {
          $("#mm-media-preview img").css("border", this.checked ? "1px solid black" : "none");
        });
      $("input#edit-mm-media-margin")
        .change(function() {
          $("#mm-media-preview img").css("margin", this.checked ? "10px" : "0");
        });
    });
  }
};
})(jQuery);
JS
  , array('type' => 'inline', 'scope' => 'footer'));
}
